#define __UNICODE
#define WIN32_LEAN_AND_MEAN


#include <windows.h>
#include <Winsock2.h>
#include <Ws2tcpip.h>
#include <stdio.h>
#include <stdlib.h>
#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>
#include <process.h>  
#include "zlib.h"


#if defined(MAINFILE)
#define FTNDLL __declspec(dllexport)
#else
#define FTNDLL __declspec(dllimport)
#endif


typedef struct {
	HANDLE hHeap;
	size_t CurrentAllocated;
	size_t CurrentSize;
	char * lpBuffer;
	} StringBuffer, *lpStringBuffer;

typedef struct {
	HANDLE hHeap;
	size_t CurrentAllocated;
	size_t CurrentSize;
	wchar_t * lpBuffer;
	} WStringBuffer, *lpWStringBuffer;

typedef struct {
	HANDLE hHeap;
	size_t CurrentAllocated;
	size_t CurrentSize;
	unsigned int * lpBuffer;
	} DwordBuffer, *lpDwordBuffer;


typedef struct {
	unsigned short origNode;
	unsigned short destNode;
	unsigned short year;
	unsigned short month;
	unsigned short day;
	unsigned short hour;
	unsigned short minute;
	unsigned short second;
	unsigned short baud;
	unsigned short PktVer;
	unsigned short origNet;
	unsigned short destNet;
	unsigned char ProductCodeL;
	unsigned char RevisionH;
	char password[8];
	unsigned short origZoneQ;
	unsigned short destZoneQ;
	unsigned short auxNet;
	unsigned char cw1H;
	unsigned char cw1L;
	unsigned char ProductCodeH;
	unsigned char RevisionL;
	unsigned char cw2L;
	unsigned char cw2H;	
	unsigned short origZone;
	unsigned short destZone;
	unsigned short origPoint;
	unsigned short destPoint;
	unsigned char reserved[4];
	} PktHeader,*lpPktHeader;


typedef struct {
	unsigned short MsgVer;
	unsigned short origNode;
	unsigned short destNode;
	unsigned short origNet;
	unsigned short destNet;
	unsigned short attribute;
	unsigned short cost;
	} MsgHeader,*lpMsgHeader;
	
	


	
	
typedef union 
{
	unsigned __int64 FullAddr;
	struct
	{
		WORD zone;
		WORD net;
		WORD node;
		WORD point;
	};
} FTNAddr, *lpFTNAddr;





typedef struct tagMsgHdr
{
	char FromName[36];
	char ToName[36];
	char Subj[72];
	char DateTime[20];
	unsigned short timesRead;
	unsigned short DestNode;
	unsigned short OrigNode;
	unsigned short cost;
	unsigned short OrigNet;
	unsigned short DestNet;
	unsigned short DestZone;
	unsigned short OrigZone;
	unsigned short DestPoint;
	unsigned short OrigPoint;
	unsigned short ReplyTo;
	unsigned short Attribute;
	unsigned short NextReply;
} MsgHdr,*lpMsgHdr;




typedef struct tagNetmailMessage
{
	union
	{unsigned int FromLinkID;
	unsigned int MessageID;
	};
	wchar_t * FromName;
	wchar_t * ToName;
	FTNAddr FromAddr;
	FTNAddr ToAddr;
	SQL_TIMESTAMP_STRUCT CreateTime;
	wchar_t * Subject;
	wchar_t * MsgId;
	wchar_t * ReplyTo;
	wchar_t * MsgText;
	union
	{
		unsigned int flags;
		struct
		{
			unsigned recv:1;
			unsigned killsent:1;
			unsigned pvt:1;
			unsigned fileattach:1;
//			unsigned hold:1;
			unsigned arq:1;
			unsigned rrq:1;
			unsigned returnreq:1;
			unsigned direct:1;
//			unsigned orphan:1;
//			unsigned killfilesent:1;
			unsigned cfm:1;
			
		};
	};

	struct tagNetmailMessage *NextMsg;
} NetmailMessage, *lpNetmailMessage;


typedef struct tagNetmailMsgQueue
{
	lpNetmailMessage First;
	lpNetmailMessage Last;
	HANDLE hNewNetmailMsgEvent;
	CRITICAL_SECTION CritSect;
} NetmailMsgQueue, *lpNetmailMsgQueue;

typedef struct tagEchoMessage
{
	wchar_t * AreaName;
	unsigned int EchoAreaID;
	unsigned int FromLinkID;
	unsigned int ToMyAkaId;
	wchar_t * FromName;
	wchar_t * ToName;
	FTNAddr FromAddr;
	SQL_TIMESTAMP_STRUCT CreateTime;
	wchar_t * Subject;
	wchar_t * MsgId;
	wchar_t * ReplyTo;
	wchar_t * MsgText;
	DwordBuffer Path;
	DwordBuffer SeenBy;
	struct tagEchoMessage *NextMsg;
} EchomailMessage, *lpEchomailMessage;

typedef struct tagEchomailMsgQueue
{
	lpEchomailMessage First;
	lpEchomailMessage Last;
	HANDLE hNewEchomailMsgEvent;
	CRITICAL_SECTION CritSect;
} EchomailMsgQueue, *lpEchomailMsgQueue;

typedef struct tagGlobalConfig
{
	FTNAddr MyAddr;
	unsigned int ThreadCount;
	SQLHENV henv;
	wchar_t * ConnectionString;
	
	HANDLE hMainHeap;
	HANDLE hNetmailHeap;
	HANDLE hEchomailHeap;
	
	HANDLE hExitEvent;
	HANDLE hThreadEndEvent;

	HANDLE hLinksUpdateEvent;

	HANDLE hPktInEvent;
	HANDLE hNetmailOutEvent;
	
	HANDLE hEchomailTossEvent;
	HANDLE hEchomailOutEvent;
	HANDLE hSchedulerUpdateEvent;
	wchar_t * BadInPktDir;
	wchar_t * BadOutPktDir;
	wchar_t * BinkdPwdCfg;
	wchar_t * BinkOutboundDir;
	wchar_t * FileboxesDir;
	wchar_t * InboundDir;
	wchar_t * InsecureInPktDir;
	wchar_t * TmailPwdCfg;
	wchar_t * TmpOutboundDir;
	wchar_t * NodelistDir;
	wchar_t * UnzipCommand;
	wchar_t * ZipCommand;

	wchar_t * SoftwareName;
	wchar_t * SystemName;

	unsigned int MaxAkaID;
	FTNAddr * MyAkaTable;

	unsigned int RobotsAreaID;

	//

	
} GlobalConfig, *lpGlobalConfig;

typedef BOOL (*NETMAILROBOTPROC)(lpNetmailMessage,unsigned int,BOOL); 

typedef struct tagModuleInfo
{
	wchar_t * ModuleName;
	wchar_t * ModuleFileName;
	wchar_t * ModuleEventName;
} ModuleInfo, *lpModuleInfo;


// 
extern FTNDLL GlobalConfig cfg;

extern NETMAILROBOTPROC NetmailRobot[127];


extern NetmailMsgQueue NetmailQueue;

extern CRITICAL_SECTION NetmailRouteCritSect;


//


//main.c
//FTNDLL void init(void);


//util.c
FTNDLL wchar_t * TrimWStr(wchar_t * Str);
FTNDLL char * TrimStr(char * Str);
FTNDLL void StringBufferFreeMem(lpStringBuffer lpStrBuf);
FTNDLL void WStringBufferFreeMem(lpWStringBuffer lpStrBuf);
FTNDLL void DwordBufferFreeMem(lpDwordBuffer lpBuf);
FTNDLL void InitStringBuffer(HANDLE hHeap,lpStringBuffer lpStrBuffer);
FTNDLL void InitWStringBuffer(HANDLE hHeap,lpWStringBuffer lpStrBuffer);
FTNDLL void AllocWStringBuffer(HANDLE hHeap, lpWStringBuffer lpWStrBuffer, size_t size);
FTNDLL void InitDwordBuffer(HANDLE hHeap,lpDwordBuffer lpDWBuffer);
FTNDLL void AllocDwordBuffer(HANDLE hHeap,lpDwordBuffer lpDWBuffer,size_t Lenght);
FTNDLL void AddStrToBuffer(lpStringBuffer lpStrBuf, const char * StrToAdd);
FTNDLL void AddStr1ToBuffer(lpStringBuffer lpStrBuf, const char * lpStrToAdd,int size);
FTNDLL void AddWStrToBuffer(lpWStringBuffer lpWStrBuf, const wchar_t * WStrToAdd);
FTNDLL void AddToDwordBuffer(lpDwordBuffer lpDWBuf, unsigned int DW);
FTNDLL BOOL CheckInDwordBuffer(lpDwordBuffer lpDWBuf, unsigned int DW);
FTNDLL void SortDwordBuffer(lpDwordBuffer lpDWBuf);
FTNDLL unsigned int GetMsgIdTime(HSTMT hStmt);
FTNDLL unsigned int GetPktNumber(HSTMT hStmt);
FTNDLL unsigned int GetInt(HSTMT hStmt, wchar_t * ParamName);
FTNDLL SQL_TIMESTAMP_STRUCT GetSQLTime(void);
FTNDLL void StrToSqlDateTime(SQL_TIMESTAMP_STRUCT *result,char *str);
FTNDLL wchar_t * GetBigString(HANDLE hHeap, HSTMT hstmt,wchar_t *StrName);
FTNDLL wchar_t * GetString(HANDLE hHeap, HSTMT hstmt,wchar_t *StrName);
FTNDLL void TimeToMessageStr(char * Str, SQL_TIMESTAMP_STRUCT * st);
FTNDLL void AddLogEntry(wchar_t * Message);
void WritePktHeader(HANDLE hFile, lpFTNAddr lpFromAka, lpFTNAddr lpToAddr, char * Pwd);
void ClosePktFile(HANDLE hFile);




//netmail.c
void InitNetmailQueue();
FTNDLL lpNetmailMessage CreateNewNetmailMsg(FTNAddr Addr, HSTMT hstmt);
FTNDLL void PostReplyMsg(lpNetmailMessage lpMsg, HSTMT hstmt,wchar_t *FromName, wchar_t *Subj, wchar_t *MsgText,unsigned char isDirect, unsigned char isKillSent);
FTNDLL void AddToNetmailQueue(lpNetmailMessage NewMsg);
FTNDLL void NetmailFreeMem(lpNetmailMessage lpNetmailMsg);
DWORD WINAPI NetmailTosserThread(LPVOID param);

//echomail.c
void InitEchomailQueue();
FTNDLL void PostEchomailMessage(HSTMT hstmt, unsigned int EchoAreaID, wchar_t * FromName, wchar_t * ToName, wchar_t * Subj, lpWStringBuffer lpMsgTextBuf);
FTNDLL void AddToEchomailQueue(lpEchomailMessage NewMsg);
FTNDLL void EchomailFreeMem(lpEchomailMessage lpEchomailMsg);
int CheckDupes(HSTMT hstmt, lpEchomailMessage lpMsg);
int GetEchoMsgStatus(HSTMT hstmt, lpEchomailMessage lpMsg);
DWORD WINAPI EchomailTosserThread(LPVOID param);


//PktInTosser
DWORD WINAPI PktInTosserThread(LPVOID param);

//ConfigsThread
DWORD WINAPI ConfigsThread(LPVOID param);

//NetmailOut
DWORD WINAPI NetmailOutThread(LPVOID param);
void LogSessionAndSendNetmailToLink(lpFTNAddr lpLinkAddr, unsigned char SoftwareCode);

//ModuleThread
DWORD WINAPI ModuleThread(LPVOID param);

// echomail out
DWORD WINAPI EchomailTossThread(LPVOID param);
DWORD WINAPI EchomailOutThread(LPVOID param);

//busy flags
DWORD WINAPI BusyPipesServerThread(LPVOID param);

//scheduler
DWORD WINAPI SchedulerThread(LPVOID param);
